<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', './');
require PUN_ROOT.'include/common.php';


if ($pun_user['g_read_board'] == '0')
	message($lang_common['No view']);

// Load the post.php/edit.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/post.php';


$id = isset($_GET['id']) ? intval($_GET['id']) : 0;
if ($id < 1)
	message($lang_common['Bad request']);


// Fetch some info about the post, the topic and the forum
$result = $db->query('SELECT f.id AS fid, f.forum_name, f.moderators, f.approval, f.redirect_url, fp.post_replies, fp.post_topics, t.id AS tid, t.subject, t.posted, t.first_post_id, t.closed, p.poster, p.poster_id, p.message, p.hide_smilies FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON t.id=p.topic_id INNER JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.id='.$id) or error('Unable to fetch post info', __FILE__, __LINE__, $db->error());
if (!$db->num_rows($result))
	message($lang_common['Bad request']);

$cur_post = $db->fetch_assoc($result);

// Sort out who the moderators are and if we are currently a moderator (or an admin)
$mods_array = ($cur_post['moderators'] != '') ? unserialize($cur_post['moderators']) : array();
$is_admmod = ($pun_user['g_id'] == PUN_ADMIN || ($pun_user['g_moderator'] == '1' && array_key_exists($pun_user['username'], $mods_array))) ? true : false;

$can_edit_subject = ($id == $cur_post['first_post_id'] && (($pun_user['g_edit_subjects_interval'] == '0' || (time() - $cur_post['posted']) < $pun_user['g_edit_subjects_interval']) || $is_admmod)) ? true : false;

// Do we have permission to edit this post?
if (($pun_user['g_edit_posts'] == '0' ||
	$cur_post['poster_id'] != $pun_user['id'] ||
	$cur_post['closed'] == '1') &&
	!$is_admmod)
	message($lang_common['No permission']);


// Start with a clean slate
$errors = array();

if (isset($_POST['form_sent']))
{
	// If it is a topic it must contain a subject
	if ($can_edit_subject)
	{
		$subject = trim($_POST['req_subject']);

		if ($subject == '')
			$errors[] = $lang_post['No subject'];
		else if (pun_strlen($subject) > 70)
			$errors[] = $lang_post['Too long subject'];
		else if ($pun_config['p_subject_all_caps'] == '0' && strtoupper($subject) == $subject && !$pun_user['is_admmod'])
			$subject = ucwords(strtolower($subject));
	}

	// Clean up message from POST
	$message = pun_linebreaks(trim($_POST['req_message']));

	if ($message == '')
		$errors[] = $lang_post['No message'];
	else if (strlen($message) > 65535)
		$errors[] = $lang_post['Too long message'];
	else if ($pun_config['p_message_all_caps'] == '0' && strtoupper($message) == $message && !$pun_user['is_admmod'])
		$message = ucwords(strtolower($message));

	// Validate BBCode syntax
	if ($pun_config['p_message_bbcode'] == '1' && strpos($message, '[') !== false && strpos($message, ']') !== false)
	{
		require PUN_ROOT.'include/parser.php';
		$message = preparse_bbcode($message, $errors);
	}


	$hide_smilies = isset($_POST['hide_smilies']) ? intval($_POST['hide_smilies']) : 0;
	if ($hide_smilies != '1') $hide_smilies = '0';

	// Did everything go according to plan?
	if (empty($errors) && !isset($_POST['preview']))
	{
		$edited_sql = (!isset($_POST['silent']) || !$is_admmod) ? $edited_sql = ', edited='.time().', edited_by=\''.$db->escape($pun_user['username']).'\'' : '';

		if ($db_type != 'mysql' && $db_type != 'mysqli')
			require PUN_ROOT.'include/search_idx.php';

		// Does the post need to be approved?
		$approved = ($cur_post['approval'] == 1 && $pun_user['g_posts_approved'] == 0 && !$is_admmod) ? 0 : 1;

		if ($can_edit_subject)
		{
			// Update the topic and any redirect topics
			$db->query('UPDATE '.$db->prefix.'topics SET subject=\''.$db->escape($subject).'\' WHERE id='.$cur_post['tid'].' OR moved_to='.$cur_post['tid']) or error('Unable to update topic', __FILE__, __LINE__, $db->error());

			// We changed the subject, so we need to take that into account when we update the search words
			if ($db_type != 'mysql' && $db_type != 'mysqli')
				update_search_index('edit', $id, $message, $subject);
		}
		else if ($db_type != 'mysql' && $db_type != 'mysqli')
			update_search_index('edit', $id, $message);

		// Update the post
		$db->query('UPDATE '.$db->prefix.'posts SET message=\''.$db->escape($message).'\', approved=\''.$approved.'\', hide_smilies=\''.$hide_smilies.'\''.$edited_sql.' WHERE id='.$id) or error('Unable to update post', __FILE__, __LINE__, $db->error());

		redirect(pun_link($pun_url['post'], $id), $lang_post['Edit redirect']);
	}
}


// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])),
	array($cur_post['forum_name'], pun_link($pun_url['forum'], $cur_post['fid'])),
	array($cur_post['subject'], pun_link($pun_url['topic'], $cur_post['tid'])),
	$lang_post['Edit post']
);

define('PUN_PAGE', 'postedit');
require PUN_ROOT.'header.php';

?>
<div class="a-section a-main">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

	<div class="b-postform">
		<div class="c-section c-form">
<?php

// If preview selected and there are no errors
if (isset($_POST['preview']) && empty($errors))
{
	require_once PUN_ROOT.'include/parser.php';
	$preview_message = parse_message(trim($_POST['req_message']), $hide_smilies);

?>
			<div class="panel">
				<div class="form-intro">
					<h2><span><?php echo $lang_post['Post preview'] ?></span></h2>
				</div>
				<div class="post-entry">
					<div class="entry-content">
					<?php echo $preview_message."\n" ?>
					</div>
				</div>
			</div>
			<div class="divider"><hr /></div>
<?php

}

?>
			<form id="afocus" method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['edit'], $id) ?>">
			<div class="panel">
<?php

// Setup post help/navigation links
$post_help = array();

if ($pun_config['p_message_bbcode'] == '1') $post_help[] = '<li><a href="'.pun_link($pun_url['help'], 'bbcode').'" onclick="window.open(this.href); return false;" title="'.$lang_common['New window warn'].'">'.$lang_common['BBCode help'].'</a></li>';
if ($pun_config['p_message_img_tag'] == '1') $post_help[] = '<li><a href="'.pun_link($pun_url['help'], 'img').'" onclick="window.open(this.href); return false;" title="'.$lang_common['New window warn'].'">'.$lang_common['img tag help'].'</a></li>';
if ($pun_config['o_smilies'] == '1') $post_help[] = '<li><a href="'.pun_link($pun_url['help'], 'smilies').'" onclick="window.open(this.href); return false;" title="'.$lang_common['New window warn'].'">'.$lang_common['Smilies help'].'</a></li>';

if (!empty($post_help))
{

?>
				<div class="form-intro">
					<ul class="optionlist">
						<?php echo implode("\n\t\t\t\t\t\t", $post_help)."\n" ?>
					</ul>
				</div>
<?php

}

if ($is_admmod)
	echo "\t\t\t\t".'<input type="hidden" name="csrf_token" value="'.$pun_user['csrf_token'].'" />'."\n";

// If there were any errors, show them
if (!empty($errors))
{

?>
				<div class="form-warn">
					<p><strong><?php echo $lang_common['Warning'] ?></strong> <?php echo $lang_post['Post errors'] ?></p>
					<ul class="warnlist">
<?php

	while (list(, $cur_error) = each($errors))
		echo "\t\t\t\t\t\t".'<li><span>'.$cur_error.'</span></li>'."\n";

?>
					</ul>
				</div>
<?php

}

$fld_count = 0;

?>
				<p id="req-msg" class="req-warn"><?php printf($lang_common['Required warn'], '<em class="req-text">'.$lang_common['Required mark'].'<span> '.$lang_common['Required'].'</span></em>') ?></p>
				<fieldset>
					<legend><span><?php echo $lang_post['Edit post legend'] ?></span></legend>
					<input type="hidden" name="form_sent" value="1" />
<?php if ($can_edit_subject): ?>					<p class="longfield required">
						<label for="fld<?php echo ++ $fld_count ?>"><?php echo $lang_common['Subject'] ?> <em class="req-text"><?php echo $lang_common['Required mark'].' <span>'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><input id="fld<?php echo $fld_count ?>" type="text" name="req_subject" size="80" maxlength="70" value="<?php echo htmlspecialchars(isset($_POST['req_subject']) ? $_POST['req_subject'] : $cur_post['subject']) ?>" /></span>
					</p>
<?php endif ?>					<p class="areafield required">
						<label for="fld<?php echo ++ $fld_count ?>"><?php echo $lang_common['Message'] ?>  <em class="req-text"><?php echo $lang_common['Required mark'].' <span>'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><textarea id="fld<?php echo $fld_count ?>" name="req_message" rows="20" cols="95"><?php echo htmlspecialchars(isset($_POST['req_message']) ? $message : $cur_post['message']) ?></textarea></span>
					</p>
<?php

$checkboxes = array();
if ($pun_config['o_smilies'] == '1')
{
	if (isset($_POST['hide_smilies']) || $cur_post['hide_smilies'] == '1')
		$checkboxes[] = '<span class="input"><input type="checkbox" id="fld'.(++ $fld_count).'" name="hide_smilies" value="1" checked="checked" /><label for="fld'.$fld_count.'">'.$lang_post['Hide smilies'].'</label></span><br />';
	else
		$checkboxes[] = '<span class="input"><input type="checkbox" id="fld'.(++ $fld_count).'" name="hide_smilies" value="1" /><label for="fld'.$fld_count.'">'.$lang_post['Hide smilies'].'</label></span><br />';
}

if ($is_admmod)
{
	if ((isset($_POST['form_sent']) && isset($_POST['silent'])) || !isset($_POST['form_sent']))
		$checkboxes[] = '<span class="input"><input type="checkbox" id="fld'.(++ $fld_count).'" name="silent" value="1" checked="checked" /><label for="fld'.$fld_count.'">'.$lang_post['Silent edit'].'</label></span><br />';
	else
		$checkboxes[] = '<span class="input"><input type="checkbox" id="fld'.(++ $fld_count).'" name="silent" value="1" /><label for="fld'.$fld_count.'">'.$lang_post['Silent edit'].'</label></span><br />';
}

if (!empty($checkboxes))
{

?>
					<fieldset class="checkset">
						<legend><span><?php echo $lang_post['Post options'] ?></span></legend>
						<?php echo implode("\n\t\t\t\t\t\t", $checkboxes)."\n"; ?>
					</fieldset>
<?php

}

?>
				</fieldset>
				<p class="formsubmit"><input type="submit" class="button" name="submit" value="<?php echo $lang_common['Submit'] ?>" accesskey="s" title="<?php echo $lang_common['Submit title'] ?>" />  <input type="submit" class="button" name="preview" value="<?php echo $lang_post['Preview'] ?>" accesskey="p" title="<?php echo $lang_post['Preview title'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

require PUN_ROOT.'footer.php';
