<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', './');
require PUN_ROOT.'include/common.php';


// If we are logged in, we shouldn't be here
if (!$pun_user['is_guest'])
{
	header('Location: '.pun_link($pun_url['index']));
	exit;
}

// Load the register.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/register.php';

// Load the register.php/profile.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/prof_reg.php';


if ($pun_config['o_regs_allow'] == '0')
	message($lang_register['No new regs']);


// User pressed the cancel button
if (isset($_GET['cancel']))
	redirect(pun_link($pun_url['index']), $lang_register['Reg cancel redirect']);


// Show the rules
else if ($pun_config['o_rules'] == '1' && !isset($_GET['agree']) && !isset($_POST['form_sent']))
{
	// Setup breadcrumbs
	$pun_page['crumbs'] = array(
		array($pun_config['o_board_title'], pun_link($pun_url['index'])),
		array($lang_common['Register'], pun_link($pun_url['register'])),
		$lang_common['Rules']
	);

	define('PUN_PAGE', 'rules');
	require PUN_ROOT.'header.php';

?>
<div class="a-section a-main">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<div class="panel">
				<div class="userbox">
					<?php echo $pun_config['o_rules_message'] ?>
				</div>
			</div>
			<form method="get" accept-charset="utf-8" action="<?php echo $base_url ?>/register.php">
			<div class="panel">
				<p><strong>In order to register you must first agree to the rules set out above</strong></p>
				<p class="formsubmit"><input type="submit" class="button" name="agree" value="<?php echo $lang_register['Agree'] ?>" /> <input type="submit" class="button" name="cancel" value="<?php echo $lang_register['Cancel'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

	require PUN_ROOT.'footer.php';
}


else if (isset($_POST['form_sent']))
{
	// Check that someone from this IP didn't register a user within the last hour (DoS prevention)
	$result = $db->query('SELECT 1 FROM '.$db->prefix.'users WHERE registration_ip=\''.get_remote_address().'\' AND registered>'.(time() - 3600)) or error('Unable to fetch user info', __FILE__, __LINE__, $db->error());

	if ($db->num_rows($result))
		message('A new user was registered with the same IP address as you within the last hour. To prevent registration flooding, at least an hour has to pass between registrations from the same IP. Sorry for the inconvenience.');

	$username = trim($_POST['req_username']);
	$email1 = strtolower(trim($_POST['req_email1']));

	if ($pun_config['o_regs_verify'] == '1')
	{
		$email2 = strtolower(trim($_POST['req_email2']));

		$password1 = random_key(8, true);
		$password2 = $password1;
	}
	else
	{
		$password1 = trim($_POST['req_password1']);
		$password2 = trim($_POST['req_password2']);
	}

	// Convert multiple whitespace characters into one (to prevent people from registering with indistinguishable usernames)
	$username = preg_replace('#\s+#s', ' ', $username);

	// Validate username and passwords
	if (pun_strlen($username) < 2)
		message($lang_prof_reg['Username too short']);
	else if (pun_strlen($username) > 25)	// This usually doesn't happen since the form element only accepts 25 characters
		message($lang_common['Bad request']);
	else if (pun_strlen($password1) < 4)
		message($lang_prof_reg['Pass too short']);
	else if ($password1 != $password2)
		message($lang_prof_reg['Pass not match']);
	else if (strtolower($username) == 'guest' || strtolower($username) == strtolower($lang_common['Guest']))
		message($lang_prof_reg['Username guest']);
	else if (preg_match('/[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}/', $username))
		message($lang_prof_reg['Username IP']);
	else if ((strpos($username, '[') !== false || strpos($username, ']') !== false) && strpos($username, '\'') !== false && strpos($username, '"') !== false)
		message($lang_prof_reg['Username reserved chars']);
	else if (preg_match('#\[b\]|\[/b\]|\[u\]|\[/u\]|\[i\]|\[/i\]|\[color|\[/color\]|\[quote\]|\[quote=|\[/quote\]|\[code\]|\[/code\]|\[img\]|\[/img\]|\[url|\[/url\]|\[email|\[/email\]#i', $username))
		message($lang_prof_reg['Username BBCode']);

	// Check username for any censored words
	if ($pun_config['o_censoring'] == '1' && censor_words($username) != $username)
		message($lang_register['Username censor']);

	// Check for username dupe
	$dupe = check_username_dupe($username);
	if ($dupe !== false)
		message(sprintf($lang_prof_reg['Username dupe'], htmlspecialchars($dupe)));

	// Validate e-mail
	require PUN_ROOT.'include/email.php';

	if (!is_valid_email($email1))
		message($lang_common['Invalid e-mail']);
	else if ($pun_config['o_regs_verify'] == '1' && $email1 != $email2)
		message($lang_register['E-mail not match']);

	// Check if it's a banned e-mail address
	if (is_banned_email($email1))
	{
		if ($pun_config['p_allow_banned_email'] == '0')
			message($lang_prof_reg['Banned e-mail']);

		$banned_email = true;	// Used later when we send an alert e-mail
	}
	else
		$banned_email = false;

	// Check if someone else already has registered with that e-mail address
	$dupe_list = array();

	$result = $db->query('SELECT username FROM '.$db->prefix.'users WHERE email=\''.$email1.'\'') or error('Unable to fetch user info', __FILE__, __LINE__, $db->error());
	if ($db->num_rows($result))
	{
		if ($pun_config['p_allow_dupe_email'] == '0')
			message($lang_prof_reg['Dupe e-mail']);

		while ($cur_dupe = $db->fetch_assoc($result))
			$dupe_list[] = $cur_dupe['username'];
	}

	// Make sure we got a valid language string
	if (isset($_POST['language']))
	{
		$language = preg_replace('#[\.\\\/]#', '', $_POST['language']);
		if (!file_exists(PUN_ROOT.'lang/'.$language.'/common.php'))
			message($lang_common['Bad request']);
	}
	else
		$language = $pun_config['o_default_lang'];

	$save_pass = (!isset($_POST['save_pass']) || $_POST['save_pass'] != '1') ? '0' : '1';

	$email_setting = intval($_POST['email_setting']);
	if ($email_setting < 0 || $email_setting > 2) $email_setting = 1;

	// Insert the new user into the database. We do this now to get the last inserted id for later use.
	$now = time();

	$intial_group_id = ($pun_config['o_regs_verify'] == '0') ? $pun_config['o_default_user_group'] : PUN_UNVERIFIED;
	$salt = random_key(12);
	$password_hash = sha1($salt.sha1($password1));

	// Add the user
	$db->query('INSERT INTO '.$db->prefix.'users (username, group_id, password, email, email_setting, save_pass, timezone, language, style, registered, registration_ip, last_visit, salt) VALUES(\''.$db->escape($username).'\', '.$intial_group_id.', \''.$password_hash.'\', \''.$email1.'\', '.$email_setting.', '.$save_pass.', \''.$db->escape($_POST['timezone']).'\' , \''.$db->escape($language).'\', \''.$pun_config['o_default_style'].'\', '.$now.', \''.get_remote_address().'\', '.$now.', \''.$db->escape($salt).'\')') or error('Unable to create user', __FILE__, __LINE__, $db->error());
	$new_uid = $db->insert_id();


	// If we previously found out that the e-mail was banned
	if ($banned_email && $pun_config['o_mailing_list'] != '')
	{
		$mail_subject = 'Alert - Banned e-mail detected';
		$mail_message = 'User \''.$username.'\' registered with banned e-mail address: '.$email1."\n\n".'User profile: '.pun_link($pun_url['user'], $new_uid)."\n\n".'-- '."\n".'Forum Mailer'."\n".'(Do not reply to this message)';

		pun_mail($pun_config['o_mailing_list'], $mail_subject, $mail_message);
	}

	// If we previously found out that the e-mail was a dupe
	if (!empty($dupe_list) && $pun_config['o_mailing_list'] != '')
	{
		$mail_subject = 'Alert - Duplicate e-mail detected';
		$mail_message = 'User \''.$username.'\' registered with an e-mail address that also belongs to: '.implode(', ', $dupe_list)."\n\n".'User profile: '.pun_link($pun_url['user'], $new_uid)."\n\n".'-- '."\n".'Forum Mailer'."\n".'(Do not reply to this message)';

		pun_mail($pun_config['o_mailing_list'], $mail_subject, $mail_message);
	}

	// Should we alert people on the admin mailing list that a new user has registered?
	if ($pun_config['o_regs_report'] == '1')
	{
		$mail_subject = 'Alert - New registration';
		$mail_message = 'User \''.$username.'\' registered in the forums at '.$base_url.'/'."\n\n".'User profile: '.pun_link($pun_url['user'], $new_uid)."\n\n".'-- '."\n".'Forum Mailer'."\n".'(Do not reply to this message)';

		pun_mail($pun_config['o_mailing_list'], $mail_subject, $mail_message);
	}

	// Must the user verify the registration or do we log him/her in right now?
	if ($pun_config['o_regs_verify'] == '1')
	{
		// Load the "welcome" template
		$mail_tpl = trim(file_get_contents(PUN_ROOT.'lang/'.$pun_user['language'].'/mail_templates/welcome.tpl'));

		// The first row contains the subject
		$first_crlf = strpos($mail_tpl, "\n");
		$mail_subject = trim(substr($mail_tpl, 8, $first_crlf-8));
		$mail_message = trim(substr($mail_tpl, $first_crlf));

		$mail_subject = str_replace('<board_title>', $pun_config['o_board_title'], $mail_subject);
		$mail_message = str_replace('<base_url>', $base_url.'/', $mail_message);
		$mail_message = str_replace('<username>', $username, $mail_message);
		$mail_message = str_replace('<password>', $password1, $mail_message);
		$mail_message = str_replace('<login_url>', pun_link($pun_url['login']), $mail_message);
		$mail_message = str_replace('<board_mailer>', sprintf($lang_common['Forum mailer'], $pun_config['o_board_title']), $mail_message);

		pun_mail($email1, $mail_subject, $mail_message);

		message(sprintf($lang_register['Reg e-mail'], '<a href="mailto:'.$pun_config['o_admin_email'].'">'.$pun_config['o_admin_email'].'</a>'));
	}

	pun_setcookie($new_uid, $password_hash, ($save_pass != '0') ? $now + 31536000 : 0);

	redirect('index.php', $lang_register['Reg complete']);
}


// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])),
	$lang_common['Register']
);

define('PUN_PAGE', 'register');
require PUN_ROOT.'header.php';

?>
<div class="a-section a-main">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form id="afocus" method="post" accept-charset="utf-8" action="<?php echo $base_url ?>/register.php?action=register">
			<div class="panel">
				<div class="form-intro">
					<h2 class="prefix"><span><?php echo $lang_register['Registration info'] ?></span></h2>
					<p><?php echo $lang_register['Register intro'] ?></p>
				</div>
				<p id="req-msg" class="req-warn"><?php printf($lang_common['Required warn'], '<em class="req-text">'.$lang_common['Required mark'].'<span> '.$lang_common['Required'].'</span></em>') ?></p>
				<fieldset class="textset" id="fs1">
					<legend><span><?php echo $lang_register['Reg required legend'] ?></span></legend>
					<input type="hidden" name="form_sent" value="1" />
					<p class="field required">
						<label for="fs1f1"><?php echo $lang_common['Username'] ?> <em class="req-text"><?php echo $lang_common['Required mark'].' <span class="acchide">'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><input type="text" id="fs1f1" name="req_username" size="25" maxlength="25" /> <em><?php echo $lang_prof_reg['Username info'] ?></em></span>
					</p>
<?php if ($pun_config['o_regs_verify'] == '0'): ?>					<p class="field required">
						<label for="fs1f2"><?php echo $lang_common['Password'] ?> <em class="req-text"><?php echo $lang_common['Required mark'].' <span class="acchide">'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><input type="password" id="fs1f2" name="req_password1" size="16" /> <em><?php echo $lang_prof_reg['Pass info'] ?></em></span>
					</p>
					<p class="field required">
						<label for="fs1f3"><?php echo $lang_prof_reg['Confirm pass'] ?> <em class="req-text"><?php echo $lang_common['Required mark'].' <span class="acchide">'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><input type="password" id="fs1f3" name="req_password2" size="16" /> <em><?php echo $lang_prof_reg['Pass info'] ?></em></span>
					</p>
<?php endif; ?>					<p class="field required">
						<label for="fs1f4"><?php echo $lang_prof_reg['E-mail address'] ?> <em class="req-text"><?php echo $lang_common['Required mark'].' <span class="acchide">'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><input type="text" id="fs1f4" name="req_email1" size="35" maxlength="80" /></span>
					</p>
<?php if ($pun_config['o_regs_verify'] == '1'): ?>					<p class="field required">
						<label for="fs1f5"><?php echo $lang_register['Confirm e-mail'] ?> <em class="req-text"><?php echo $lang_common['Required mark'].' <span class="acchide">'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><input type="text" id="fs1f5" name="req_email2" size="35" maxlength="80" /></span>
					</p>
					<div class="form-warn">
						<p><strong><?php echo $lang_common['Important'] ?></strong> <?php echo $lang_register['E-mail info'] ?></p>
<?php endif; ?>				</fieldset>
				<fieldset class="textset" id="fs2">
					<legend><span><?php echo $lang_prof_reg['Local legend'] ?></span></legend>
<?php

		$languages = array();
		$d = dir(PUN_ROOT.'lang');
		while (($entry = $d->read()) !== false)
		{
			if ($entry != '.' && $entry != '..' && is_dir(PUN_ROOT.'lang/'.$entry) && file_exists(PUN_ROOT.'lang/'.$entry.'/common.php'))
				$languages[] = $entry;
		}
		$d->close();

		// Only display the language selection box if there's more than one language available
		if (count($languages) > 1)
		{
			natcasesort($languages);

?>
					<p class="field">
						<label for="fs2f1"><?php echo $lang_prof_reg['Language'] ?></label><br />
						<span class="input"><select id="fs2f1" name="language">
<?php

			while (list(, $temp) = @each($languages))
			{
				if ($pun_config['o_default_lang'] == $temp)
					echo "\t\t\t\t\t\t\t".'<option value="'.$temp.'" selected="selected">'.$temp.'</option>'."\n";
				else
					echo "\t\t\t\t\t\t\t".'<option value="'.$temp.'">'.$temp.'</option>'."\n";
			}

?>
						</select></span>
					</p>
<?php

		}

?>
					<p class="field">
						<label for="fs2f2"><?php echo $lang_prof_reg['Timezone'] ?></label><br />
						<span class="input"><select id="fs2f2" name="timezone">
							<option value="-12"<?php if ($pun_config['o_default_timezone'] == -12 ) echo ' selected="selected"' ?>>-12</option>
							<option value="-11"<?php if ($pun_config['o_default_timezone'] == -11) echo ' selected="selected"' ?>>-11</option>
							<option value="-10"<?php if ($pun_config['o_default_timezone'] == -10) echo ' selected="selected"' ?>>-10</option>
							<option value="-9.5"<?php if ($pun_config['o_default_timezone'] == -9.5) echo ' selected="selected"' ?>>-9.5</option>
							<option value="-9"<?php if ($pun_config['o_default_timezone'] == -9 ) echo ' selected="selected"' ?>>-09</option>
							<option value="-8.5"<?php if ($pun_config['o_default_timezone'] == -8.5) echo ' selected="selected"' ?>>-8.5</option>
							<option value="-8"<?php if ($pun_config['o_default_timezone'] == -8 ) echo ' selected="selected"' ?>>-08 PST</option>
							<option value="-7"<?php if ($pun_config['o_default_timezone'] == -7 ) echo ' selected="selected"' ?>>-07 MST</option>
							<option value="-6"<?php if ($pun_config['o_default_timezone'] == -6 ) echo ' selected="selected"' ?>>-06 CST</option>
							<option value="-5"<?php if ($pun_config['o_default_timezone'] == -5 ) echo ' selected="selected"' ?>>-05 EST</option>
							<option value="-4"<?php if ($pun_config['o_default_timezone'] == -4 ) echo ' selected="selected"' ?>>-04 AST</option>
							<option value="-3.5"<?php if ($pun_config['o_default_timezone'] == -3.5) echo ' selected="selected"' ?>>-3.5</option>
							<option value="-3"<?php if ($pun_config['o_default_timezone'] == -3 ) echo ' selected="selected"' ?>>-03 ADT</option>
							<option value="-2"<?php if ($pun_config['o_default_timezone'] == -2 ) echo ' selected="selected"' ?>>-02</option>
							<option value="-1"<?php if ($pun_config['o_default_timezone'] == -1) echo ' selected="selected"' ?>>-01</option>
							<option value="0"<?php if ($pun_config['o_default_timezone'] == 0) echo ' selected="selected"' ?>>00 GMT</option>
							<option value="1"<?php if ($pun_config['o_default_timezone'] == 1) echo ' selected="selected"' ?>>+01 CET</option>
							<option value="2"<?php if ($pun_config['o_default_timezone'] == 2 ) echo ' selected="selected"' ?>>+02</option>
							<option value="3"<?php if ($pun_config['o_default_timezone'] == 3 ) echo ' selected="selected"' ?>>+03</option>
							<option value="3.5"<?php if ($pun_config['o_default_timezone'] == 3.5 ) echo ' selected="selected"' ?>>+03.5</option>
							<option value="4"<?php if ($pun_config['o_default_timezone'] == 4 ) echo ' selected="selected"' ?>>+04</option>
							<option value="4.5"<?php if ($pun_config['o_default_timezone'] == 4.5 ) echo ' selected="selected"' ?>>+04.5</option>
							<option value="5"<?php if ($pun_config['o_default_timezone'] == 5 ) echo ' selected="selected"' ?>>+05</option>
							<option value="5.5"<?php if ($pun_config['o_default_timezone'] == 5.5 ) echo ' selected="selected"' ?>>+05.5</option>
							<option value="6"<?php if ($pun_config['o_default_timezone'] == 6 ) echo ' selected="selected"' ?>>+06</option>
							<option value="6.5"<?php if ($pun_config['o_default_timezone'] == 6.5 ) echo ' selected="selected"' ?>>+06.5</option>
							<option value="7"<?php if ($pun_config['o_default_timezone'] == 7 ) echo ' selected="selected"' ?>>+07</option>
							<option value="8"<?php if ($pun_config['o_default_timezone'] == 8 ) echo ' selected="selected"' ?>>+08</option>
							<option value="9"<?php if ($pun_config['o_default_timezone'] == 9 ) echo ' selected="selected"' ?>>+09</option>
							<option value="9.5"<?php if ($pun_config['o_default_timezone'] == 9.5 ) echo ' selected="selected"' ?>>+09.5</option>
							<option value="10"<?php if ($pun_config['o_default_timezone'] == 10) echo ' selected="selected"' ?>>+10</option>
							<option value="10.5"<?php if ($pun_config['o_default_timezone'] == 10.5 ) echo ' selected="selected"' ?>>+10.5</option>
							<option value="11"<?php if ($pun_config['o_default_timezone'] == 11) echo ' selected="selected"' ?>>+11</option>
							<option value="11.5"<?php if ($pun_config['o_default_timezone'] == 11.5 ) echo ' selected="selected"' ?>>+11.5</option>
							<option value="12"<?php if ($pun_config['o_default_timezone'] == 12 ) echo ' selected="selected"' ?>>+12</option>
							<option value="13"<?php if ($pun_config['o_default_timezone'] == 13 ) echo ' selected="selected"' ?>>+13</option>
							<option value="14"<?php if ($pun_config['o_default_timezone'] == 14 ) echo ' selected="selected"' ?>>+14</option>
						</select></span>
						<span class="input"><input type="checkbox" id="fs2f3" /> <label for="fs2f3"><?php echo $lang_prof_reg['DST label'] ?></label></span>
					</p>
				</fieldset>
				<fieldset class="radioset" id="fs3">
					<legend><span><?php echo $lang_prof_reg['E-mail visibility legend'] ?></span></legend>
					<p class="field">
						<span class="input"><input type="radio" id="fs3f1" name="email_setting" value="0" /> <label for="fs3f1"><?php echo $lang_prof_reg['E-mail setting 1'] ?></label></span><br />
						<span class="input"><input type="radio" id="fs3f2" name="email_setting" value="1" checked="checked" /> <label for="fs3f2"><?php echo $lang_prof_reg['E-mail setting 2'] ?></label></span><br />
						<span class="input"><input type="radio" id="fs3f3" name="email_setting" value="2" /> <label for="fs3f3"><?php echo $lang_prof_reg['E-mail setting 3'] ?></label></span><br />
					</p>
				</fieldset>
				<fieldset class="checkset" id="fs4">
					<legend><span><?php echo $lang_prof_reg['Persistent login legend'] ?></span></legend>
					<p class="field">
						<span class="input"><input type="checkbox" id="fs4f1" name="save_pass" value="1" checked="checked" /> <label for="fs4f1"><?php echo $lang_prof_reg['Save user/pass'] ?></label></span>
					</p>
				</fieldset>
				<p class="formsubmit"><input type="submit" class="button" name="register" value="<?php echo $lang_register['Register'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

require PUN_ROOT.'footer.php';
