<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', './');
require PUN_ROOT.'include/common.php';


if ($pun_user['g_read_board'] == '0')
	message($lang_common['No view']);

// Load the viewforum.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/forum.php';


$id = isset($_GET['id']) ? intval($_GET['id']) : 0;
if ($id < 1)
	message($lang_common['Bad request']);


// Fetch some info about the forum
$query = array(
	'SELECT'	=> 'f.forum_name, f.redirect_url, f.moderators, f.approval, f.num_topics, f.sort_by, fp.post_topics',
	'FROM'		=> 'forums AS f',
	'JOINS'		=> array(
		array(
			'LEFT JOIN'		=> 'forum_perms AS fp',
			'ON'			=> '(fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].')'
		)
	),
	'WHERE'		=> '(fp.read_forum IS NULL OR fp.read_forum=1) AND f.id='.$id
);

($hook = get_hook('vf_qr_get_forum_info')) ? eval($hook) : null;
$result = $db->query_build($query) or error('Unable to fetch forum info', __FILE__, __LINE__, $db->error());
if (!$db->num_rows($result))
	message($lang_common['Bad request']);

$cur_forum = $db->fetch_assoc($result);

// Is this a redirect forum? In that case, redirect!
if ($cur_forum['redirect_url'] != '')
{
	header('Location: '.$cur_forum['redirect_url']);
	exit;
}

// Sort out who the moderators are and if we are currently a moderator (or an admin)
$mods_array = array();
if ($cur_forum['moderators'] != '')
	$mods_array = unserialize($cur_forum['moderators']);

$is_admmod = ($pun_user['g_id'] == PUN_ADMIN || ($pun_user['g_moderator'] == '1' && array_key_exists($pun_user['username'], $mods_array))) ? true : false;

// Get topic/forum tracking data
if (!$pun_user['is_guest'])
	$tracked_topics = get_tracked_topics();

// Determine the topic offset (based on $_GET['p'])
$num_pages = ceil($cur_forum['num_topics'] / $pun_user['disp_topics']);

$p = (!isset($_GET['p']) || $_GET['p'] <= 1 || $_GET['p'] > $num_pages) ? 1 : $_GET['p'];
$start_from = $pun_user['disp_topics'] * ($p - 1);

// Generate paging links
$main_links = '<div class="pagelinks">'.$lang_common['Pages'].': '.paginate($num_pages, $p, $pun_url['forum'], array($id, sef_friendly($cur_forum['forum_name']))).'</div>';

// Can we or can we not post new topics?
if (($cur_forum['post_topics'] == '' && $pun_user['g_post_topics'] == '1') || $cur_forum['post_topics'] == '1' || $is_admmod)
	$main_links .= "\n\t\t".'<div class="postlink"><a class="newpost" href="'.pun_link($pun_url['new_topic'], $id).'"><span>'.$lang_forum['Post topic'].'</span></a></div>';


// Navigation links for header and page numbering for title/meta description
if ($p < $num_pages)
{
	$pun_page['nav'][] = '<link rel="last" href="'.pun_sublink($pun_url['forum'], $pun_url['page'], $num_pages, array($id, sef_friendly($cur_forum['forum_name']))).'" title="'.$lang_common['Page'].' '.$num_pages.'" />';
	$pun_page['nav'][] = '<link rel="next" href="'.pun_sublink($pun_url['forum'], $pun_url['page'], ($p + 1), array($id, sef_friendly($cur_forum['forum_name']))).'" title="'.$lang_common['Page'].' '.($p + 1).'" />';
}
if ($p > 1)
{
	$pun_page['nav'][] = '<link rel="prev" href="'.pun_sublink($pun_url['forum'], $pun_url['page'], ($p - 1), array($id, sef_friendly($cur_forum['forum_name']))).'" title="'.$lang_common['Page'].' '.($p - 1).'" />';
	$pun_page['nav'][] = '<link rel="first" href="'.pun_link($pun_url['forum'], array($id, sef_friendly($cur_forum['forum_name']))).'" title="'.$lang_common['Page'].' 1" />';
	$pun_page['page'] = $p;
}

// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])),
	$cur_forum['forum_name']
);

define('PUN_ALLOW_INDEX', 1);
define('PUN_PAGE', 'viewforum');
require PUN_ROOT.'header.php';

// Fetch list of topics to display on this page

// The basic view forum query
$query = array(
	'SELECT'	=> 't.id, t.poster, t.subject, t.posted, t.first_post_id, t.last_post, t.last_post_id, t.last_poster, t.num_views, t.num_replies, t.closed, t.sticky, t.moved_to',
	'FROM'		=> 'topics AS t',
	'WHERE'		=> 't.forum_id='.$id,
	'ORDER BY'	=> 'sticky DESC, '.(($cur_forum['sort_by'] == '1') ? 'posted' : 'last_post').' DESC',
	'LIMIT'		=> $start_from.', '.$pun_user['disp_topics']
);

// With "has posted" indication
if (!$pun_user['is_guest'] && $pun_config['o_show_dot'] == '1')
{
	$query['SELECT'] .= ', p.poster_id AS has_posted';
	$query['JOINS'][] = array(
		'LEFT JOIN'	=> 'posts AS p',
		'ON'		=> 't.id=p.topic_id AND p.poster_id='.$pun_user['id']
	);

	if ($db_type == 'sqlite')
	{
		$query['WHERE'] = 't.id IN(SELECT id FROM '.$db->prefix.'topics WHERE forum_id='.$id.' ORDER BY sticky DESC, '.(($cur_forum['sort_by'] == '1') ? 'posted' : 'last_post').' DESC LIMIT '.$start_from.', '.$pun_user['disp_topics'].')';
		$query['ORDER BY'] = 't.sticky DESC, t.last_post DESC';
	}

	if ($db_type == 'pgsql')
		$query['GROUP BY'] = 't.id, t.subject, t.poster, t.posted, t.first_post_id, t.last_post, t.last_post_id, t.last_poster, t.num_views, t.num_replies, t.closed, t.sticky, t.moved_to, p.poster_id';
	else
		$query['GROUP BY'] = 't.id';
}

// Add approval stuff to the query if needed
if ($cur_forum['approval'] == 1)
{
	// Highlight any topics containing unapproved posts for moderators
	if ($is_admmod)
	{
		$query['SELECT'] .= ', a.approved AS has_unapproved';
		$query['JOINS'][] = array(
			'LEFT JOIN'	=> 'posts AS a',
			'ON'		=> 't.id=a.topic_id AND a.approved=0'
		);

		// We need the group by statement here
		if ($db_type == 'pgsql')
			$query['GROUP BY'] = 't.id, t.subject, t.poster, t.posted, t.first_post_id, t.last_post, t.last_post_id, t.last_poster, t.num_views, t.num_replies, t.closed, t.sticky, t.moved_to, p.poster_id';
		else
			$query['GROUP BY'] = 't.id';
	}
	// Hide any topics where the first post is a) unapproved and b) not made by the current user
	else
	{
		$query['SELECT'] .= ', a.approved, a.poster_id';
		$query['JOINS'][] = array(
			'LEFT JOIN'	=> 'posts AS a',
			'ON'		=> 't.first_post_id=a.id'
		);

		// With "has posted" indication and PostgreSQL, we need to add to the group by
		if (!$pun_user['is_guest'] && $pun_config['o_show_dot'] == '1' && $db_type == 'pgsql')
			$query['GROUP BY'] .= ', a.approved, a.poster_id';

		$query['WHERE'] .=  ' AND (a.approved=1 '.($pun_user['is_guest'] ? '' : 'OR a.poster_id='.$pun_user['id']).')';
	}
}

($hook = get_hook('vf_qr_get_topics')) ? eval($hook) : null;
$result = $db->query_build($query) or error('Unable to fetch topic list', __FILE__, __LINE__, $db->error());

?>
<div id="pun-main1" class="a-section a-main paged">

	<h1 class="main-title"><span><a class="permalink" href="<?php echo pun_link($pun_url['forum'], array($id, sef_friendly($cur_forum['forum_name']))) ?>" rel="bookmark" title="<?php echo $lang_forum['Permalink forum'] ?>"><?php echo htmlspecialchars($cur_forum['forum_name']) ?></a></span></h1>

	<div class="main-head">
		<?php echo $main_links."\n" ?>
	</div>
<?php

// If there are topics in this forum
if ($db->num_rows($result))
{

?>
	<div id="forum<?php echo $id ?>" class="b-section b-forum">
		<div class="c-section c-table">
			<table cellspacing="0" summary="<?php echo $lang_forum['Table summary'].htmlspecialchars($cur_forum['forum_name']); if (!$pun_user['is_guest'] && $pun_config['o_show_dot'] == '1') echo ' '.$lang_forum['Dot explain'] ?>">
			<thead>
				<tr>
					<th class="tcl" scope="col"><?php echo $lang_common['Topic'] ?></th>
					<th class="tc2" scope="col"><?php echo $lang_common['Replies'] ?></th>
<?php
	if ($pun_config['o_topic_views'] == '1')
		echo "\t\t\t\t\t".'<th class="tc3" scope="col">'.$lang_forum['Views'].'</th>'."\n";
?>
					<th class="tcr" scope="col"><?php echo $lang_common['Last post'] ?></th>
				</tr>
			</thead>
			<tbody class="hasicon">
<?php

	$row_count = $start_from;
	while ($cur_topic = $db->fetch_assoc($result))
	{
		++$row_count;
		$row_style = ($row_count % 2 != 0) ? 'altstyle' : '';
		$icon_type = 'icon';

		if ($cur_topic['moved_to'] == null)
			$last_post = '<a href="'.pun_link($pun_url['post'], $cur_topic['last_post_id']).'">'.format_time($cur_topic['last_post']).'</a> <span class="byuser">'.$lang_common['by'].'&#160;'.htmlspecialchars($cur_topic['last_poster']).'</span>';
		else
			$last_post = $lang_forum['Moved'];

		if ($pun_config['o_censoring'] == '1')
			$cur_topic['subject'] = censor_words($cur_topic['subject']);

		if ($cur_topic['moved_to'] != null)
			$subject = $lang_forum['Moved'].': <a href="'.pun_link($pun_url['topic'], array($cur_topic['moved_to'], sef_friendly($cur_topic['subject']))).'">'.htmlspecialchars($cur_topic['subject']).'</a> <span class="byuser">'.$lang_common['by'].'&#160;'.htmlspecialchars($cur_topic['poster']).'</span>';
		else if ($cur_topic['closed'] == '0')
			$subject = '<a href="'.pun_link($pun_url['topic'], array($cur_topic['id'], sef_friendly($cur_topic['subject']))).'">'.htmlspecialchars($cur_topic['subject']).'</a> <span class="byuser">'.$lang_common['by'].'&#160;'.htmlspecialchars($cur_topic['poster']).'</span>';
		else
		{
			$subject = '<span class="closedtopic">'.$lang_common['Closed'].'</span> <a href="'.pun_link($pun_url['topic'], array($cur_topic['id'], sef_friendly($cur_topic['subject']))).'">'.htmlspecialchars($cur_topic['subject']).'</a> <span class="byuser">'.$lang_common['by'].'&#160;'.htmlspecialchars($cur_topic['poster']).'</span>';
			$row_style .= ' iclosed';
		}

		if (!$pun_user['is_guest'] && $cur_topic['last_post'] > $pun_user['last_visit'] && (!isset($tracked_topics[$cur_topic['id']]) || $tracked_topics[$cur_topic['id']] < $cur_topic['last_post']) && $cur_topic['moved_to'] == null)
		{
			$row_style .= ' inew';
			$icon_type = 'icon inew';
			$subject_new_posts = '<span class="newposts">[&#160;<a href="'.pun_link($pun_url['topic_new_posts'], array($cur_topic['id'], sef_friendly($cur_topic['subject']))).'" title="'.$lang_common['New posts info'].'">'.$lang_common['New posts'].'</a>&#160;]</span>';
		}
		else
			$subject_new_posts = null;


		if ($cur_topic['sticky'] == '1')
		{
			$subject = '<span class="stickytopic">'.$lang_forum['Sticky'].'</span> '.$subject;
			$row_style .= ' isticky';
		}

		// Should we display the dot or not? :)
		if (!$pun_user['is_guest'] && $pun_config['o_show_dot'] == '1')
		{
			if ($cur_topic['has_posted'] == $pun_user['id'])
			{
				$subject = '<em>'.$lang_forum['You posted indicator'].'</em>'.$subject;
				$row_style .= ' iposted';
			}
		}

		$num_pages_topic = ceil(($cur_topic['num_replies'] + 1) / $pun_user['disp_posts']);

		if ($num_pages_topic > 1)
			$subject_multipage = '[&#160;<span class="acchide">'.$lang_common['Pages'].':&#160;</span>'.paginate($num_pages_topic, -1, $pun_url['topic'], array($cur_topic['id'], sef_friendly($cur_topic['subject']))).'&#160;]';
		else
			$subject_multipage = null;

		// Should we show the "New posts" and/or the multipage links?
		if (!empty($subject_new_posts) || !empty($subject_multipage))
		{
			$subject .= '&#160; '.(!empty($subject_new_posts) ? $subject_new_posts : '');
			$subject .= !empty($subject_multipage) ? ' '.$subject_multipage : '';
		}

		// Does the topic contain unapproved posted?
		$cur_topic['has_unapproved'] = (isset($cur_topic['has_unapproved'])) ? 1 : 0;

		if ($cur_topic['has_unapproved'] == 1 && $is_admmod)
			$row_style .= ' unapproved';

?>
				<tr<?php if ($row_style != '') echo ' class="'.trim($row_style).'"' ?>>
					<td class="tcl"><div class="<?php echo $icon_type ?>"><!-- --></div><?php echo '<span class="acchide">'.$row_count.'</span> '.$subject ?></td>
					<td class="tc2"><?php echo ($cur_topic['moved_to'] == null) ? $cur_topic['num_replies'] : '&#160;' ?></td>
<?php
		if ($pun_config['o_topic_views'] == '1')
			echo "\t\t\t\t\t".'<td class="tc3">'.($cur_topic['moved_to'] == null ? $cur_topic['num_views'] : '&#160;' ) .'</td>'."\n";
?>
					<td class="tcr"><?php echo $last_post ?></td>
				</tr>
<?php

	}

?>
			</tbody>
			</table>
		</div>
	</div>

<?php

// Display forum options
$main_options[] = '<li id="mo-rss" class="mo-users"><a href="'.pun_link($pun_url['forum_rss'], $id).'">'.$lang_common['RSS Feed'].'</a></li>';
$main_options[] = '<li id="mo-atom" class="mo-users"><a href="'.pun_link($pun_url['forum_atom'], $id).'">'.$lang_common['ATOM Feed'].'</a></li>';

if ($is_admmod)
	$main_options[] = '<li id="mo-moderate" class="mo-mods"><a href="'.pun_sublink($pun_url['moderate_forum'], $pun_url['page'], $p, $id).'" title="'.$lang_forum['Moderate forum info'].'">'.$lang_forum['Moderate forum'].'</a></li>';

?>
	<div class="b-section b-mainoptions">
		<div class="c-section c-any">
			<h2 class="prefix"><span><?php echo $lang_forum['Forum options'] ?></span></h2>
			<ul class="optionlist">
				<?php echo implode("\n\t\t\t\t", $main_options)."\n" ?>
			</ul>
		</div>
	</div>

<?php

}
else
{

?>
	<div  id="forum<?php echo $id ?>" class="b-section b-message">
		<div class="c-section c-any">
			<p><?php echo $lang_forum['Empty forum'] ?></p>
		</div>
	</div>

<?php

}

?>
	<div class="main-foot">
		<?php echo $main_links."\n" ?>
	</div>

</div>
<?php

$forum_id = $id;
require PUN_ROOT.'footer.php';
